/**
 * Copyright (c) 2007 Moses Gunesch, MosesSupposes.com - Donovan Adams, blog.hydrotik.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
 
package com.hydrotik.go {
	import flash.display.DisplayObject;

	import org.fuseproject.go.items.LinearGo;
	import org.papervision3d.objects.DisplayObject3D;
	
	//import flash.geom.ColorTransform;
	//import flash.filters.BlurFilter;
	//import flash.geom.Point;	

	/**
	 * A basic example of how you could build a tween on LinearGo.
	 */
	public class PapervisionTween extends LinearGo {

		// -== Public Properties ==-
		public function get width() : Number {
			return _rotationY;
		}

		public function set width(value : Number) : void {
			if (_state == STOPPED)
				_rotationY = value;
		} 		

		public function get startWidth() : Number {
			return _startRotationY;
		}

		public function set startWidth(value : Number) : void {
			if (_state == STOPPED)
				_startRotationY = value;
		} 		

		public function get target() : DisplayObject {
			return _target;
		}

		public function set target(obj : DisplayObject) : void {
			if (_state == STOPPED)
				_target = obj;
		}

		// -== Protected Properties ==-
		protected var _target : *;

		protected var _rotationY : Number;
		
		private static var _viewport:*;
		
		private static var _camera:*;
		
		private static var _scene:*;
		
		private static var _renderer : *;
		
		private var _closure : Function;

		private var debug : Function;

		protected var _startRotationY : Number;

		protected var _changeRotationY : Number;

		protected var _startProps : Object = {};

		protected var _changeProps : Object = {};

		protected var _propsTo : Object = {};
		
		
		public static function init(renderer:*, scene:*, camera:*, viewport:*):void{
			_renderer = renderer;
			_scene = scene;
			_camera = camera;
			_viewport = viewport;
		}

		// -== Public Methods ==-
		public function PapervisionTween(
					target:* = null,
					propsTo:Object = null,
					delay:Number = NaN,
					duration:Number = NaN,
					easing:Function = null,
					closure:Function = null)
		{			
			super(delay, duration, easing);
			debug = trace;
			_target = target;
			if(closure != null) addCallback(closure);
			for (var prop in propsTo) { 
				switch (prop) {
					case "alpha":
						if(_target is DisplayObject){
							_propsTo[prop] = (propsTo[prop] != undefined) ? propsTo[prop] : _target[prop];	
						}
						if(_target is DisplayObject3D){
							if(propsTo[prop] != undefined){
								_propsTo[prop] = propsTo[prop];
							}else{
								_target.extra[prop] = new Number(1);
								_propsTo[prop] = _target.extra[prop] = _target.extra[prop];
							}
						}
						break;
					default:
						_propsTo[prop] = (propsTo[prop] != undefined) ? propsTo[prop] : _target[prop];
				}
			}
		}

		override public function start( ) :Boolean {
			if (!_target) return false;
			var prop:String;
			for (prop in _propsTo) { 
				switch (prop) {
					case "alpha":
						if(_target is DisplayObject){
							_startProps[prop] = _target[prop];
						}
						if(_target is DisplayObject3D){
							trace("start: "+_target.extra[prop]);
							_startProps[prop] = _target.extra[prop];	
						}
						break;
					default:
						_startProps[prop] = _target[prop];
				}
			}
			
			if (useRelative) {
				for (prop in _propsTo) { 
				    _changeProps[prop] = _propsTo[prop];
				}
			}
			else {
				for (prop in _propsTo) { 
				    _changeProps[prop] = (_propsTo[prop] - _startProps[prop]); //_propsTo[prop];
				}
			}
			
			return (super.start());
		}
		
		//TODO add alpha tweening syntax when released in 2.0 update
		override protected function onUpdate(type:String) : void {
			for (var prop in _propsTo) { 
				switch (prop) {
					case "alpha":
						if(_target is DisplayObject3D){
							//var val:Number = super.correctValue(_startProps[prop] + _changeProps[prop] * _position);
							//_target.material.bitmap.colorTransform(_target.material.bitmap.rect, new ColorTransform(1, 1, 1, val, 0, 0, 0, 0));
							//_target.extra[prop] = val;
							debug("Current version of Pv3D does not support alpha tweening");
						}
						if(_target is DisplayObject) _target[prop] = super.correctValue(_startProps[prop] + _changeProps[prop] * _position);
						break;
					default:
						_target[prop] = super.correctValue(_startProps[prop] + _changeProps[prop] * _position);
				}
			}
			if(_target is DisplayObject3D) _renderer.renderScene(_scene,_camera,_viewport);
		}
		
	}
}
