﻿package com.hydrotik.utils {
	
    import flash.events.*;
    import flash.net.*;
	import com.hydrotik.utils.XMLLoaderEvent;
	
    public class XMLLoader implements IEventDispatcher {
		
		public static const VERSION:String = "XMLLoader 0.0.1";
		
		public static const AUTHOR:String = "Donovan Adams - donovan[(at)]hydrotik.com  - http://blog.hydrotik.com";
		
		private var dispatcher:EventDispatcher;
		
		private var _loader:URLLoader;
		
		/**
		 * XMLLoader AS3
		 *
		 * @author: Donovan Adams, E-Mail: donovan[(at)]hydrotik.com, url: http://blog.hydrotik.com/
		 * @version: 0.0.1
		 *
		 * @description XML Loading Utility.
		 *
		 *
		 * @history 10.30.2007 - 3.0 Initial port to AS3.0 (Donovan Adams)
		 *
		 * @example This example shows how to use XMLLoader in a basic application:
				<code>
					import com.hydrotik.utils.XMLLoader;
					import com.hydrotik.utils.XMLLoaderEvent;
					
					
					var xmlLoader:XMLLoader = new XMLLoader(_sAdminPath + xmlFilename);
					xmlLoader.addEventListener(XMLLoaderEvent.COMPLETE, onXMLComplete);
					xmlLoader.addEventListener(XMLLoaderEvent.ERROR, onXMLError);
					xmlLoader.addEventListener(XMLLoaderEvent.PROGRESS, onXMLProgress);
				
				
					// --== Private Methods ==--
					function onXMLError(event:XMLLoaderEvent):void {
						trace("XMLLoader error: "+event)
					}
					
					function onXMLProgress(event:XMLLoaderEvent):void {
						trace("XMLLoader progress: "+event.percentage)
					}
					
					function onXMLComplete(event:XMLLoaderEvent):void {
						xml = event.xml;
					}
				</code>
		 */ 

        public function XMLLoader(path:String) {
			dispatcher = new EventDispatcher(this);
            _loader = new URLLoader();
            configureListeners(_loader);

            var request:URLRequest = new URLRequest(path);
            try {
                _loader.load(request);
            } catch (error:Error) {
                trace("Unable to load requested document.");
				dispatchEvent(new XMLLoaderEvent(XMLLoaderEvent.ERROR));
            }
        }
		
		// --== Listeners and Handlers ==--
		
		public function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void{
			dispatcher.addEventListener(type, listener, useCapture, priority, useWeakReference);
		}
			   
		public function dispatchEvent(evt:Event):Boolean{
			return dispatcher.dispatchEvent(evt);
		}
		
		public function hasEventListener(type:String):Boolean{
			return dispatcher.hasEventListener(type);
		}
		
		public function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void{
			dispatcher.removeEventListener(type, listener, useCapture);
		}
					   
		public function willTrigger(type:String):Boolean {
			return dispatcher.willTrigger(type);
		}
		
		
		// --== Private Methods ==--
		
        private function configureListeners(dispatcher:IEventDispatcher):void {
            dispatcher.addEventListener(Event.COMPLETE, completeHandler);
            dispatcher.addEventListener(ProgressEvent.PROGRESS, progressHandler);
            dispatcher.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        }

        private function completeHandler(event:Event):void {
			dispatchEvent(new XMLLoaderEvent(XMLLoaderEvent.COMPLETE, new XML(event.target.data)));
        }


        private function progressHandler(event:ProgressEvent):void {
			dispatchEvent(new XMLLoaderEvent(XMLLoaderEvent.PROGRESS, null, event.bytesLoaded/event.bytesTotal));
        }

        private function ioErrorHandler(event:IOErrorEvent):void {
            dispatchEvent(new XMLLoaderEvent(XMLLoaderEvent.ERROR));
        }
    }
	
	
}