package org.fuseproject.go.interfaces {
	
	/**
	 * The Go manager system has not yet been refined, you can ignore this file for now....
	 * 
	 * <p>
	 * An IHandlerManager assists in the management of IPropertyHandler
	 * instances. One example is GoOverlapMonitor, which monitors and prevents 
	 * property-handling overlaps between tweens or other Go items.
	 * <p>
	 * </p>
	 * The Go system decouples manager classes so they remain compile-
	 * optional for the end user, who must explicitly register an instance 
	 * of each desired manager for use with GoEngine. To uphold this system
	 * it is extremely important that IPropertyHandler item classes do not
	 * import or make any direct references to specific IHandlerManager 
	 * classes, although it's fine for managers to import and work explicitly 
	 * with specific item datatypes such as IPostUpdatable, since the end user
	 * will only register a manager for use with their crop of items.
	 * </p>
	 * 
	 * @see IPropertyHandler
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public interface IHandlerManager 
	{
		
		/**
		 * Reserve an IPropertyHandler's target/property combinations for monitoring.
		 * 
		 * @param handler		IPropertyHandler to query
		 * @param releaseOthers	Indicates that other handlers reserving overlapping 
		 * 						properties should be released to prevent conflicts.
		 */
		function reserve(handler:IPropertyHandler, releaseOthers:Boolean=true):void;
		
		
		/**
		 * Releases an IPropertyHandler from being monitored.
		 */
		function release(handler:IPropertyHandler):void;
		
	}
}
