
package org.fuseproject.go.interfaces {
	import org.fuseproject.go.interfaces.IPropertyHandler;
	
	/**
	 * The Go manager system has not yet been refined, you can ignore this file for now....
	 * 
	 * <p>
	 * Makes items compatible with IPostUpdatableManager classes.
	 * </p>
	 * <p>
	 * Instructions: 
	 * <ol><li>Add a private or protected Array to your class, for storing 
	 * references to IPostUpdatableManager instances and implement the two 
	 * listener methods in this interface.</li>
	 * <li>It is the item's responsibility to call postUpdate(this) on each 
	 * IPostUpdatableManager in the Array, as the final step in its update 
	 * process.</li>
	 * </ol>
	 * </p>
	 * 
	 * @see IPostUpdatableManager
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public interface IPostUpdatable extends IPropertyHandler {
		
		/**
		 * Push the listener into an Array. After each update, cycle through 
		 * the Array and call postUpdate(this) on each item.
		 */
		function addPostUpdateListener(listener:IPostUpdatableManager):void;
		
		/**
		 * Splice the listener out of the IPostUpdatableManager Array. 
		 * (Don't skip this step, it's important for memory management.)
		 */
		function removePostUpdateListener(listener:IPostUpdatableManager):void;
		
	}
}
