
package org.fuseproject.go.interfaces {
	import org.fuseproject.go.interfaces.IHandlerManager;
	
	/**
	 * The Go manager system has not yet been refined, you can ignore this file for now....
	 * 
	 * <p>
	 * Use in conjunction with IPostUpdatable items to Elevate management 
	 * capabilities to include a tightly-synced "post-update" callback 
	 * during item play.
	 * </p>
	 * 
	 * <p>
	 * Instructions: 
	 * <ol><li>In your manager's reserve method, check whether the 
	 * handler is IPostUpdatable. If so, call addPostUpdateListener(this)
	 * on the handler.</li>
	 * <li>The handler is then responsible for calling your instance's 
	 * postUpdate method and passing a reference to itself just after it 
	 * performs each update on its engine pulse. (If your manager prefers 
	 * to execute a postUpdate after all its child items are finished 
	 * udpating, simply increment a counter variable.)</li>
	 * <li>In your managers's release method, do the same type-check and 
	 * call removePostUpdateListener(this) on the handler. Don't skip this
	 * step as it is important for memory management.</li>
	 * </ol>
	 * </p>
	 * 
	 * <p>
	 * This architecture keeps GoEngine's udpate process streamlined and 
	 * makes postUpdate duties into an optional management feature. Since all 
	 * items can declare their pulseInterval (potentially differing from each 
	 * other), it also ensures that postUpdate is synced with each 
	 * item as tightly as possble. A callback system is used because the 
	 * standard AS3 event model is far too slow for this context of live 
	 * runtime synchronization. 
	 * </p>
	 * 
	 * @see IPostUpdatable
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public interface IPostUpdatableManager extends IHandlerManager {
		
		/**
		 * IPostUpdatable items must call this method on all 
		 * IPostUpdatableManager instances that have previously
		 * called addPostUpdateListener on them, as the final step
		 * in their update process.
		 */
		function postUpdate(item:IPostUpdatable):void;
	}
}
