package org.fuseproject.go.interfaces {
	
	// TODO: Rethink the manager system, update docs.
	
	/**
	 * The Go manager system has not yet been refined, you can ignore this file for now....
	 * 
	 * <p>Implement this interface for Go items that you wish to make compatible
	 * with IHandlerManager classes.
	 * </p>
	 * 
	 * <p>
	 * The Go system decouples manager classes so they remain compile-
	 * optional for the end user, who must explicitly register an instance 
	 * of each desired manager for use with GoEngine. To uphold this system
	 * it is <b>extremely important that IPropertyHandler item classes do not
	 * import or make any direct references to specific IHandlerManager 
	 * classes.</b>
	 * </p>
	 * 
	 * @see IHandlerManager
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public interface IPropertyHandler 
	{
		
		/**
		 * IPropertyHandler requirement: Return one or more
		 * targets actively being handled by the item. 
		 * 
		 * Note: In the same way LinearGo prevents users from
		 * setting values like duration during play, you should
		 * similarly lock an item's targets from changing during
		 * play. (GoOverlapMonitor relies on targetes being the same at
		 * play end as they are at play start.)
		 */
		function get targets() : Array;
		
		
		/**
		 * IPropertyHandler requirement: Return an Array of
		 * all properties actively being handled by the item.
		 * This list is used to check against other active 
		 * IPropertyHandler items using their isHandling method.
		 */
		function get activeProperties () : Array;
		
		
		/**
		 * IPropertyHandler requirement: Return true if any of the 
		 * property strings passed in directly coincides with any 
		 * property being actively handled.
		 * 
		 * <i>This does NOT mean any property the item handles, just
		 * ones the instance is currently tweening or setting.</i>
		 * It is also not limited to standard DisplayObject properties, 
		 * but should include any custom property names the item defines 
		 * like 'saturation', 'volume', and so on.
		 * 
		 * <b>Indirect matching:</b> 
		 * You must be sure to check for indirect, as well as direct 
		 * matches. This is very important and can at times require some 
		 * creative thought on your part.
		 * 
		 * <ul>
		 * <li>Overlap, like width vs. scaleX: These would certainly conflict 
		 * if two different Go items were allowed to handle them at once on 
		 * the same target. Overlaps might not always be this obvious,
		 * so it's important to think creatively.</li>
		 * 
		 * <li>Multi-property groups, like bezier + x + y: If the item is 
		 * setting multiple properties at once for a single result, such as
		 * a bezier-curve tween that operates on x and y and may also define
		 * a custom property like 'bezier', be sure to return true if any of 
		 * those properties are passed in.</li>
		 * 
		 * <li>Multi-property groups with overlap, like size & scale: The
		 * most complex situation in which both of the above occurs. Consider
		 * a class with custom 'scale' and 'size' properties that tween
		 * scaleX/scaleY and width/height. Overlap occurs between entire 
		 * groups of properties: scaleX/width/scale/size & scaleY/height/scale/size. 
		 * You must check whether each property passed in conflicts with this 
		 * item's active properties using those groupings to check for any 
		 * indirect match.</li>
		 * </ul>
		 * 
		 * Despite these potential complexities, always try to write isHandling
		 * to be as effiecient as possible to minimize impact on system
		 * performance. (All active items are checked when any item is started.)
		 * 
		 * @param properties	Whether the item is actively handling any of the 
		 * 						properties directly - or any other property that
		 * 						indirectly overlaps with - any of the properties 
		 * 						passed in.
		 */
		function isHandling (properties : Array) : Boolean;
		
		/**
		 * TODO: Rethink this method...
		 * 
		 * @param targets		Null indicates "all targets", or certain target Objects may be specified.
		 * @param properties	Null indicates "all properties", or certain property Strings may be specified.
		 * @return				Success
		 */
		function release (targets:Array=null, properties:Array=null) : Boolean;
		
	}
}