package org.fuseproject.go.interfaces {
	
	/**
	 * Makes any object compatible with GoEngine.
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public interface IUpdatable 
	{
		
		/**
		 * Perform updates on a pulse.
		 * 
		 * @param currentTime	A clock time that should be used instead of getTimer
		 * 						in performing update calculations. (The value is usually 
		 * 						not more than a couple milliseconds different than getTimer
		 * 						but using it tightly syncs all items in the timer group
		 * 						and can make a perceptible difference.)
		 */
		function update (currentTime : Number) : void;
		
		
		/**
		 * Defines the pulse on which update is called. 
		 * 
		 * @return	A number of milliseconds for Timer-based updates or GoEngine.ENTER_FRAME (-1)  
		 * 			for updates synced to the Flash Player's framerate. 
		 */
		function get pulseInterval() : int;
	}
}
