package org.fuseproject.go.managers {
	import flash.utils.Dictionary;
	
	import org.fuseproject.go.interfaces.IHandlerManager;
	import org.fuseproject.go.interfaces.IPropertyHandler;

	// TODO: This portion of the project has been shelved until tween/group/sequence is working...

	/**
	 * The Go manager system has not yet been refined, you can ignore this file for now....
	 * 
	 * <p>
	 * An optional manager that prevents property-handling overlap 
	 * between Go items (e.g. two tween instances trying to operate on
	 * the same Sprite's x property). 
	 * </p>
	 * <p>
	 * </p>
	 * To use: <code>GoEngine.addManager(new GoOverlapMonitor());</code>
	 * 
	 * {In the game of Go, a superko is a rule that prevents a potentially
	 *  infinite competition - ko - over the same space.}
	 *  
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public class GoOverlapMonitor implements IHandlerManager
	{
		/**
		 * This manager uses a Dictionary with target objects as the index
		 * values and an Array of handlers currently handling the target as
		 * the stored values.
		 */
		private var handlers : Dictionary = new Dictionary(true);
		
		/**
		 * Sets an IPropertyHandler as reserving its target/property combinations.
		 * 
		 * @param handler		IPropertyHandler to reserve
		 * @param releaseOthers	Indicates that other handlers reserving overlapping 
		 * 						properties should be released to prevent conflicts.
		 */
		public function reserve(handler:IPropertyHandler, releaseOthers:Boolean=true):void
		{
			var targs:Array = handler.targets;
			var props:Array = handler.activeProperties;
			var toRelease:Array = [];
				
			if (!targs || targs.length==0 || !props || props.length==0)
				return;
			
			/* The dictionary is keyed by target objects. Cycle through
			 * the targets handled by the incoming handler (there usually 
			 * will only be one), then inspect the handlers currently assigned.
			 */
			for each (var target:Object in targs)
			{
				if (!handlers[ target ]) { // The incoming handler is the only one working with this target.
					handlers[ target ] = new Array(handler);
//					trace("GoOverlapMonitor reserved: "+handler);
					continue; 
				}
				
				if (!releaseOthers)
					continue;
				
				var handlersToTest:Array = (handlers[ target ] as Array);
				
				/* Each target object in the dictionary contains a Dictionary of
				 * IPropertyHandler instances being tracked. The same IPropertyHandler
				 * might be referenced in several places in the dictionary.
				 */
				for each (var handlerToTest:IPropertyHandler in handlersToTest) 
				{
					if (handlerToTest == handler) // very unlikely: the same handler was already reserved for this target, abort.
						return;
					
					/* Test each new property being handled on this target 
					 * against existing handlers in the nested dictionary.
					 * If an overlap is reported, release the existing handler.
					 */
					for each (var propToTest:String in props) 
					{
						
						// TODO: rewrite this.. actually this whole class needs to be tested & spruced up!
						
						if (handlerToTest.isHandling(props) == true) {
							toRelease.push(handlerToTest);
						}
					}
				}
			}
			
			for each (var handlerToRelease:IPropertyHandler in toRelease)
				release( handlerToRelease );
		}
		
		/**
		 * Releases an IPropertyHandler from being monitored.
		 * 
		 * TODO: Most handlers will use release to call stop() which in turn will result in another release call here. One solution might be to have reserve call release, and assume the user will remove the item from GoEngine which will in turn call release here, but that's iffy... 
		 */
		public function release(handler:IPropertyHandler):void
		{
			/* Assumption: Go items will not allow their
			 * targets to be changed while they're running. 
			 */
			var targs:Array = handler.targets;
			
			for each (var target:Object in targs)
			{
				try {
					var handlersForTarget:Array = (handlers[ target ] as Array);
//					trace("GoOverlapMonitor release: cycle through "+handlersForTarget.length+" handlers for "+target+"...");
					if (handlers[ target ]) {
						var i:int = handlersForTarget.indexOf(handler);
						if (i>-1) {
//							trace("GoOverlapMonitor release "+handlersForTarget[i]+" for "+target);
							(handlersForTarget[i] as IPropertyHandler).release(); // This manager treats handlersForTarget as blocked target/property units, so no target/property info is sent in this call.
							handlersForTarget.splice(i, 1);
							if (handlersForTarget.length==0) {
//								trace("GoOverlapMonitor released all handlers for target" + target);
								delete handlers[ target ]; // TODO: Will this fuck up the loop if an item gets deleted from the dictionary midstream? You can mess up array loops that way for sure... might want to store an array of items to release.
							}
						}
					}
				}
				catch (e:Error) {
					trace("GoOverlapMonitor.release: "+e.message);
				}
			}
		}
		
	}
}