﻿
package org.fuseproject.go.util {
	import org.fuseproject.go.events.SequenceEvent;

	/**
	 * A PlayableGroup wrapper for playable items in a sequence step. Dispatches 
	 * SequenceEvent.ADVANCE when all items have dispatched either STOP or COMPLETE.
	 * 
	 * @see org.fuseproject.go.util.customadvance.OnConditionTrue#
	 * @see org.fuseproject.go.util.customadvance.OnDurationComplete#
	 * @see org.fuseproject.go.util.customadvance.OnEventComplete#
	 * @see org.fuseproject.go.util.customadvance.OnPlayableComplete#
	 * @see org.fuseproject.go.util.SequenceCA#
	 * @see org.fuseproject.go.util.SequenceStepCA#
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public class SequenceStep extends PlayableGroup {
		
		/**
		 * Constructor. See PlayableGroup
		 * @see org.fuseproject.go.util.PlayableGroup#PlayableGroup
		 */
		public function SequenceStep(...items) : void {
			super((items[ 0 ] is Array) ? items[ 0 ] : items);
		}
		
		/**
		 * Internal handler for group completion, overridden to dispatch ADVANCE
		 */
		override protected function complete() : void {
			if (super._listeners==0) {
				dispatchEvent(new SequenceEvent(SequenceEvent.ADVANCE));
			}
			super.complete();
		}
	}
}
