
package org.fuseproject.go.util.customadvance {
	import org.fuseproject.go.GoEngine;
	import org.fuseproject.go.interfaces.IUpdatable;
	import org.fuseproject.go.items.GoItem;

	/**
	 * A custom advance type that triggers when a callback returns true.
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public class OnConditionTrue extends SequenceAdvance implements IUpdatable {
		
		// -== Public Properties ==-
		
		/**
		 * The pulse on which to call the callback function. Defaults to 
		 * GoItem.defaultPulseInterval if not specified.
		 */
		public function get pulseInterval() : int {
			if (isNaN(_pulse))
				_pulse = GoItem.defaultPulseInterval;
			return _pulse;
		}

		// -== Protected Properties ==-
		
		protected var _function : Function;
		protected var _pulse : Number;

		// -== Public Methods ==-
		
		/**
		 * @param callbackThatReturnsBoolean	Any function that returns a Boolean value
		 * @param pulseInterval					The pulse on which to call the callback function, which defaults to
		 * 										GoItem.defaultPulseInterval if not specified.
		 */
		public function OnConditionTrue(callbackThatReturnsBoolean: Function, pulseInterval:Number=NaN) : void {
			super();
			_function = callbackThatReturnsBoolean;
			_pulse = pulseInterval;
		}
		
		override public function start() : Boolean {
			GoEngine.addItem(this);
			return super.start(); // sets state & returns true.
		}
		
		override public function stop() : Boolean {
			GoEngine.removeItem(this);
			return super.stop(); // sets state & returns true.
		}
		
		override public function pause() : Boolean {
			if (super.pause()==false)
				return false;
			GoEngine.removeItem(this);
			return true;
		}
		
		override public function resume() : Boolean {
			if (super.resume()==false)
				return false;
			GoEngine.addItem(this);
			return true;
		}
		
		public function update(currentTime : Number) : void {
			if (_function()===true)
				super.dispatchAdvance();
		}
	}
}
