
package org.fuseproject.go.util.customadvance {
	import flash.utils.getTimer;
	
	import org.fuseproject.go.GoEngine;
	import org.fuseproject.go.interfaces.IUpdatable;
	import org.fuseproject.go.items.GoItem;

	/**
	 * A custom advance type that triggers after a specific duration has completed.
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public class OnDurationComplete extends SequenceAdvance implements IUpdatable {
		
		// -== Public Properties ==-
		
		/**
		 * The duration after which advance should occur.
		 */
		public function get duration() : Number {
			return _duration;
		}
		
		/**
		 * The pulse used to monitor the duration. Defaults to GoItem.defaultPulseInterval 
		 * if not specified. 
		 * 
		 * <p>(Note that this system is more accurate than flash.utils.Timer, especially for 
		 * pause/resume.)</p>
		 */
		public function get pulseInterval() : int {
			if (isNaN(_pulse))
				_pulse = GoItem.defaultPulseInterval;
			return _pulse;
		}
		
		// -== Protected Properties ==-
		
		protected var _duration : Number;
		protected var _pulse : Number;
		protected var _pauseTime : Number;
		protected var _startTime : int;
		
		// -== Public Methods ==-
		
		/**
		 * @param seconds	The duration after which advance should occur.
		 * @param pulseInterval	The pulse used to monitor the duration. Defaults to 
		 * 						GoItem.defaultPulseInterval if not specified.
		 */
		public function OnDurationComplete(seconds:Number, pulseInterval:Number=NaN) {
			super();
			_duration = (isNaN(seconds) ? 0 : Math.max(seconds, 0));
			_pulse = pulseInterval;
		}
		
		override public function start() : Boolean {
			_startTime = getTimer();
			_pauseTime = NaN;
			GoEngine.addItem(this);
			return super.start(); // sets state & returns true.
		}
		
		override public function stop() : Boolean {
			GoEngine.removeItem(this);
			return super.stop(); // sets state & returns true.
		}
		
		override public function pause() : Boolean {
			if (super.pause()==false)
				return false;
			_pauseTime = getTimer();
			GoEngine.removeItem(this);
			return true;
		}
		
		override public function resume() : Boolean {
			if (super.resume()==false)
				return false;
			_startTime = (getTimer() - (_pauseTime - _startTime)); 
			GoEngine.addItem(this);
			return true;
		}
		
		public function update(currentTime : Number) : void {
			if (currentTime >= _startTime + _duration*1000) {
				super.dispatchAdvance();
			}
		}
	}
}
