﻿
package org.fuseproject.go.util.customadvance {
	import flash.events.Event;
	import flash.events.IEventDispatcher;

	/**
	 * A custom advance type that triggers when any event is dispatched by any IEventDispatcher host. 
	 * 
	 * <p>If the event requires filtering or custom handling, you can optionally set up a custom handler 
	 * that accepts the event object as an input and returns true if all conditions are met for advancing 
	 * the sequence.</p>
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public class OnEventComplete extends SequenceAdvance {

		// -== Protected Properties ==-
		
		protected var _host : IEventDispatcher;
		protected var _type : String;
		protected var _customHandler : Function;

		// -== Public Methods ==-
		
		/**
		 * @param dispatcher	Any object that dispatches the event.
		 * @param type			The event type to listen for
		 * @param customHanderThatReturnsBoolean	Optionally you may specify a custom event handler which should
		 * 											accept an event input and return true once all conditions are met.
		 */
		public function OnEventComplete(dispatcher:IEventDispatcher, type:String, customHanderThatReturnsBoolean:Function=null) : void {
			super();
			_host = dispatcher;
			_type = type;
			_customHandler = customHanderThatReturnsBoolean;
		}
		
		override public function start() : Boolean {
			_host.addEventListener(_type, dispatchAdvance);
			return super.start(); // sets state & returns true.
		}
		
		override public function stop() : Boolean {
			_host.removeEventListener(_type, dispatchAdvance);
			return super.stop(); // sets state & returns true.
		}
		
		// -== Protected Methods ==-
		
		override protected function dispatchAdvance(event:Event=null) : void {
			if (_customHandler!=null) {
				try {
					if (_customHandler(event)===true)
						super.dispatchAdvance(event);
				}
				catch (e:Error) {
					throw e;
				}
				return;
			}
			super.dispatchAdvance();
		}
	}
}
