
package org.fuseproject.go.util.customadvance {
	import org.fuseproject.go.events.GoEvent;
	import org.fuseproject.go.interfaces.IPlayable;

	/**
	 * A custom advance type that triggers when any playable item (presumably one item
	 * in the step) dispatches STOP or COMPLETE.
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public class OnPlayableComplete extends SequenceAdvance {

		// -== Public Properties ==-
		
		public function set item(item : IPlayable) : void {
			if (_state==STOPPED)
				_item = item;
		}
		public function get item() : IPlayable {
			return _item;
		}
		
		// -== Public Methods ==-
		
		protected var _item : IPlayable;
		
		/**
		 * @param item		Any playable item that dispatches STOP or COMPLETE,  
		 * 					normally a child item in the step using this custom advance.
		 */
		public function OnPlayableComplete(item : IPlayable) : void {
			super();
			_item = item;
		}
		
		override public function start() : Boolean {
			_item.addEventListener(GoEvent.STOP, super.dispatchAdvance);
			_item.addEventListener(GoEvent.COMPLETE, super.dispatchAdvance);
			return super.start(); // sets state & returns true.
		}
		
		override public function stop() : Boolean {
			_item.removeEventListener(GoEvent.STOP, super.dispatchAdvance);
			_item.removeEventListener(GoEvent.COMPLETE, super.dispatchAdvance);
			return super.stop(); // sets state & returns true.
		}
	}
}
