
package org.fuseproject.go.util.customadvance {
	import flash.events.Event;
	
	import org.fuseproject.go.PlayableBase;
	import org.fuseproject.go.events.SequenceEvent;

	/**
	 * Subclasses should call the <code>dispatchAdvance</code> method when 
	 * the sequence should advance to its next step. It is mandatory that
	 * custom advance types dispatch this event one time, although each class
	 * may define its own conditions for when this event occurs.
	 * 
	 * @eventType org.fuseproject.go.events.SequenceEvent.ADVANCE
	 */
	[Event(name="ADVANCE", type="org.fuseproject.go.events.SequenceEvent")]

	/**
	 * Base class for other custom advance types, does nothing on its own.
	 * 
	 * @see org.fuseproject.go.util.customadvance.OnConditionTrue#
	 * @see org.fuseproject.go.util.customadvance.OnDurationComplete#
	 * @see org.fuseproject.go.util.customadvance.OnEventComplete#
	 * @see org.fuseproject.go.util.customadvance.OnPlayableComplete#
	 * 
	 * @author Moses Gunesch / mosessupposes.com (c) all rights reserved
	 */
	public class SequenceAdvance extends PlayableBase {

		// -== Protected Methods ==-
		
		/**
		 * Call this method from subclasses to trigger advance, only once per play cycle.
		 * @param e	Allows method to be used as an event handler.
		 */
		protected function dispatchAdvance(e:Event=null) : void {
			stop();
			dispatchEvent(new SequenceEvent(SequenceEvent.ADVANCE));
		}
	}
}
